// Copyright (c) 2003-2010 Christopher M. Kohlhoff
// Modifications (c) 2011 by Guilherme Maciel Ferreira / Intra2net AG
//
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)
#ifndef ICMP_ECHO_REQUEST_MESSAGE_H
#define ICMP_ECHO_REQUEST_MESSAGE_H

#include <stdint.h>

#include <istream>
#include <ostream>

#include "icmp/icmpmessage.h"
#include "host/messagepayload.h"

//-----------------------------------------------------------------------------
// IcmpEchoRequestMessage
//-----------------------------------------------------------------------------
//
// ICMP Echo Request Message Format
//
// 0               8               16                             31
// +---------------+---------------+------------------------------+      ---
// |               |               |                              |       ^
// |     type      |     code      |          checksum            |       |
// |               |               |                              |       |
// +---------------+---------------+------------------------------+    8 bytes
// |                               |                              |       |
// |          identifier           |       sequence number        |       |
// |                               |                              |       v
// +-------------------------------+------------------------------+      ---
// |                                                              |
// |                           data (n bytes)                     |
// |                                                              |
// +-------------------------------+------------------------------+
//
//-----------------------------------------------------------------------------

class IcmpEchoRequestMessage : public IcmpMessage
{
public:
    IcmpEchoRequestMessage();
    virtual ~IcmpEchoRequestMessage();

    virtual Icmpv4Type get_type_v4() const;
    virtual void set_type_v4( const Icmpv4Type type );

    virtual Icmpv6Type get_type_v6() const;
    virtual void set_type_v6( const Icmpv6Type type );

    virtual uint8_t get_code() const;
    virtual void set_code( const uint8_t code );

    virtual uint16_t get_checksum() const;
    virtual void set_checksum( const uint16_t checksum );

    virtual uint16_t get_identifier() const;
    virtual void set_identifier( const uint16_t identifier );

    virtual uint16_t get_sequence_number() const;
    virtual void set_sequence_number( const uint16_t sequence_number );

    virtual std::istream& read( std::istream &is );
    virtual std::ostream& write( std::ostream &os ) const;

private:
    /// packet payload object
    MessagePayload Payload;

};

#endif // ICMP_ECHO_REQUEST_MESSAGE_H
